import logging 
import sys
import time 
import threading

import _fake_parent
from dlp_sender import Sender

if __name__ == "__main__":

    UPDATE_PERIOD = 2
    DEFAULT_SLEEP_UNTIL_PRINT = 3

    logging.basicConfig()
    logger = logging.getLogger(__name__)
    #logger.setLevel(logging.WARNING)
    #logger.setLevel(logging.INFO)
    logger.setLevel(logging.DEBUG)

    def start_print_after(sender, filename, pause):
        time.sleep(pause)
        with open(filename, 'rb') as f:
            print('Sending file to print: ' + filename)
            sender.gcodes(f)

    fp = _fake_parent.FakeParent()
    usb_info = {'VID': '0000', 'PID': '0000', 'SNR': '0000', 'COM': '/dev/null', 'EMU': True, 'EXP_TIME': 1}
    profiles = fp.parent.user_login.profiles
    print(f'Got {len(profiles)} possible profiles')
    print(sys.argv)
    cmd_args = sys.argv[1:]
    if len(cmd_args) > 0:
        print_filename = cmd_args[0]
    else:
        print_filename = None
    if len(cmd_args) > 1:
        print_pause = float(cmd_args[1])
    else:
        print_pause = DEFAULT_SLEEP_UNTIL_PRINT
    for profile in profiles:
        #if profile['sender'] == __name__.replace('.py', ''):
        if profile['sender'] == 'dlp_sender':
            profile = dict(profile)
            break
    else:
        print('Unable to get any profile for DLP sender')
        sys.exit(1)
    s = Sender(fp, usb_info, profile)
    if print_filename:
        tr = threading.Thread(target=start_print_after, args=(s, print_filename, print_pause))
        tr.start()
    try:
        while not fp.stop_flag:
            print('Progress percent: ', s.get_percent())
            time.sleep(UPDATE_PERIOD)
    except KeyboardInterrupt:
        fp.stop_flag = True
    s.close()
