import logging 
import os
import sys
import time

from gcodes_buffer import *


class FakePrintInterface:

    def register_error(self, *args, **kwargs):
        print(f"Error: {args} {kwargs}")

    def set_total_gcodes(self, total_gcode_lines):
       print(f"Total gcodes: {total_gcode_lines} ")

#TODO add zip file to the test
def test_gcodes_buffer():

    FILENAME = "_gcode_buffer_test_file.gcodes"
    NUMBER_OF_LINES = 1024 * 1024
    #NUMBER_OF_LINES = 1024 * 1024 * 128
    #LOOP_SLEEP = 0.0001
    LOOP_SLEEP = 0
    LONG_SLEEP_AFTER_NLINES = 100
    BACKWARD_GET_AFTER_NLINES = 1000

    root = logging.getLogger()
    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    root.addHandler(handler)
    gb = GcodesBuffer()
    #gb = GcodesBuffer(parent_printer_interface=FakePrintInterface(), blocking_count=False)
    start_time = time.monotonic()
    silent_mode = "-s" in sys.argv
    if os.path.isfile(FILENAME):
        print("Gcodes file %s already exists - using it" % FILENAME)
    else:
        print("Creating gcodes file %s with %d lines..." % (FILENAME, NUMBER_OF_LINES))
        with open(FILENAME, "w") as f:
            for i in range(0, NUMBER_OF_LINES):
                f.write(str(i) + "\n")
            #f.writelines([str(i) + "\n" for i in range(0, NUMBER_OF_LINES)])
        print("...done. Creation time: %ss" % (time.monotonic() - start_time))
    with open(FILENAME, "rb") as f:
        gb.open(f)
        #gb = f.read().strip().split(b"\n")
        gb.insert(0, b"-1")
        index = 0
        print("Open time:", time.monotonic() - start_time)
        time.sleep(2)
        run_flag = True
        line = ""
        last_backwards_get = 0
        avg_access_time = 0.0
        max_access_time = 0.0
        max_access_line_number = 0
        maxes = []
        longs = []
        try:
            while run_flag:
                loop_start_time = time.monotonic()
                #print("Index:", index)
                #print("Line:", gb[index])
                line = gb[index]
                if not index:
                    print("First line:", line)
                elif index-1 != int(line):
                        args = (index, line)
                        raise AssertionError("Buffer error: the line should be %s, instead of %s" % args)
                        run_flag = False
                access_time = (time.monotonic() - loop_start_time) * 1000000
                if not index % LONG_SLEEP_AFTER_NLINES:
                    if not silent_mode:
                        print("Line number: %d = %s .Access time(avg, max):(%.2fus, %.2fus - line %d)" % (index,  line, avg_access_time, max_access_time, max_access_line_number))
                    time.sleep(0.01)
                if index and not index % BACKWARD_GET_AFTER_NLINES and index > last_backwards_get:
                    last_backwards_get = index
                    index -= 256
                if access_time > max_access_time:
                    max_access_time = access_time
                    max_access_line_number = index
                    maxes.append((index, access_time))
                if access_time > 200:
                    longs.append((index, access_time))
                avg_access_time = (avg_access_time + access_time) / 2.0
                #print("Access Time:", access_time)
                time.sleep(LOOP_SLEEP)
                #print("="*80)
                index += 1
        except (IndexError, KeyboardInterrupt):
            run_flag = False
        finally:
            gb.clear()
            try:
                os.remove(FILENAME)
            except:
                pass
        print("===Finished===")
        print("Lines:%d/%d" % (index+1, gb.lines_in_file))
        print("Max latency:", maxes)
        print("Long latency:", longs)
        print("File size:%dB" % gb.file_size)
        print("Total time: %s" % (time.monotonic() - start_time))
        #  run_flag = True
        #  try:
        #      while run_flag:
        #          time.sleep(0.1)
        #  except KeyboardInterrupt:
        #      pass
        #  finally:
        #      run_flag = False

if __name__ == "__main__":
    logging.basicConfig(level=logging.DEBUG)
    test_gcodes_buffer()

