#!/bin/bash

# kate: indent-pasted-text true; remove-trailing-spaces all; replace-tabs true; replace-tabs-save true; syntax Bash; word-wrap-column 200;


RED="\e[31m"
GREEN="\e[32m"
ENDCOLOR="\e[0m"
SPACER="\t\t"

WIDTH=$(tput cols)

if  [[ $1 == "--plain" ]]; then
    RED=""
    GREEN=""
    ENDCOLOR=""
    SPACER="\n"
fi

echo -e "$(uname -srm)  $(python --version)  3DPrinterOS Client $(python -c "import version; print(version.version, version.branch, version.build)")"

echo -n -e 'Board model:\t\t'
if [[ -n $(dmesg | head -n10 | grep 'Machine model' | sed -r 's/(^.+)(Machine model:\s+)(.+)/\3/g') ]] ; then
    echo -e -n "$(dmesg | head -n10 | grep 'Machine model' | sed -r 's/(^.+)(Machine model:\s+)(.+)/\3/g')"
    if [[ -n $(dmesg | grep -i "under-voltage detected") ]] ; then
        echo -n -e " ${RED}under-voltage${ENDCOLOR}"
    fi
    if vcgencmd measure_temp &> /dev/null; then
        TEMPERATURE=$(vcgencmd measure_temp | sed -r 's/^([a-z]+=)//g')
        if [[ $(echo "$TEMPERATURE" | sed -r 's/^([0-9]+)(\..+)/\1/g') -gt 65 ]] ; then
            echo -e "${RED} $TEMPERATURE${ENDCOLOR}"
        else
            echo -e "${GREEN} $TEMPERATURE${ENDCOLOR}"
        fi
    fi
else
    echo -e "${RED}detect error${ENDCOLOR}\t"
fi

echo -n -e 'SD:\t\t\t'
if [ $(df -m --output='avail' / | tail -n1) -gt 2048 ] ; then
    echo -e -n "${GREEN}$(df -m --output='avail' / | tail -n1 | xargs)${ENDCOLOR} / "
else
    echo -e -n "${RED}$(df -m --output='avail' / | tail -n1 | xargs)${ENDCOLOR} / "
fi
echo -e -n "$(df -m --output='size' / | tail -n1 | xargs) MB"
if [ $(df -m --output='size' / | tail -n1 | xargs) -gt 99999 ] ; then
    echo -e -n "\t\t"
else
    echo -e -n "\t\t\t"
fi

echo -n -e 'Ping to cloud:\t\t'
if ping cli-cloud.3dprinteros.com -c 1 &> /dev/null; then
    echo -e "${GREEN}Yes${ENDCOLOR}"
else
    echo -e "${RED}No${ENDCOLOR}"
fi

echo -n -e 'RAM:\t\t\t'
if [ $(expr $(sed -n '/^MemFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024) -gt 1024 ] ; then
    echo -e -n "${GREEN}$(expr $(sed -n '/^MemFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024)${ENDCOLOR} / "
else
    echo -e -n "${RED}$(expr $(sed -n '/^MemFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024)${ENDCOLOR} / "
fi
echo -e -n "$(expr $(sed -n '/^MemTotal:/ s/[^0-9]//gp' /proc/meminfo) / 1024) MB"
if [ $(expr $(sed -n '/^MemFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024) -gt 9999 ] ; then
echo -e -n "\t\t"
else
echo -e -n "\t\t\t"
fi

echo -n -e 'Cloud login:\t\t'
echo -e -n "$(python user_login.py --login)"
if [ $(python user_login.py --auth) == "success" ] ; then
    echo -e " ${GREEN}success${ENDCOLOR}"
else
    echo -e " ${RED}$(python user_login.py --auth)${ENDCOLOR}"
fi

echo -n -e 'Swap:\t\t\t'
if [ $(sed -n '/^SwapTotal:/ s/[^0-9]//gp' /proc/meminfo) -eq 0 ] ; then
    echo -n -e "off\t"
else
    if [ $(expr $(sed -n '/^SwapFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024) -gt 1024 ] ; then
        echo -e -n "${GREEN}$(expr $(sed -n '/^SwapFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024)${ENDCOLOR} / "
    else
        echo -e -n "${RED}$(expr $(sed -n '/^SwapFree:/ s/[^0-9]//gp' /proc/meminfo) / 1024)${ENDCOLOR} / "
    fi
    echo -e -n "$(expr $(sed -n '/^SwapTotal:/ s/[^0-9]//gp' /proc/meminfo) / 1024) MB"
fi

echo -n -e '\t\t\tHost ID:\t\t'
python http_client.py --id

echo -n -e 'Ping to 8.8.8.8:\t'
if ping 8.8.8.8 -c 1 &> /dev/null; then
    echo -e -n "${GREEN}Yes${ENDCOLOR}\t"
else
    echo -e -n "${RED}No${ENDCOLOR}\t"
fi

echo -n -e '\t\t\tNative picamera:\t'
if python -c "import picamera" &> /dev/null; then
    echo -e "${GREEN}Installed${ENDCOLOR}"
else
    echo -e "${RED}Not installed${ENDCOLOR}"
fi

echo -n -e 'PySerial:\t\t'
if python -c "import serial" &> /dev/null; then
    echo -e -n "${GREEN}Installed${ENDCOLOR}"
else
    echo -e -n "${RED}Not installed${ENDCOLOR}"
fi

echo -n -e '\t\t\tOpenCV:\t\t\t'
if python -c "import cv2" &> /dev/null; then
    echo -e "${GREEN}Installed${ENDCOLOR}"
else
    echo -e "${RED}Not installed${ENDCOLOR}"
fi
