import json
import logging
import os

import config
import paths
from base_detector import BaseDetector


class StaticDetector(BaseDetector):

    STORAGE_FILEPATH = os.path.join(paths.CURRENT_SETTINGS_FOLDER, 'static_printers.json')
    CONFIG_SECTION_TO_COPY_ON_INIT = 'static_printers'

    def __init__(self, _):
        self.logger = logging.getLogger(__class__.__name__)
        self.printers = self.load()
        if self.printers:
            self.logger.info(f'Static printers: {self.printers}')

    def load(self, section_name=None):
        if not section_name:
            section_name = self.CONFIG_SECTION_TO_COPY_ON_INIT
        try:
            return list(config.get_settings().get(section_name, {}))
        except:
            self.logger.warning('Invalid static printers section')
            return []

    def get_printers_list(self):
        self.logger.debug(f'Static printers: {self.printers}')
        return self.printers


class SavedDetector(StaticDetector):

    STORAGE_FILEPATH = os.path.join(paths.CURRENT_SETTINGS_FOLDER, 'saved_printers.json')

    def load(self, filepath):
        if os.path.isfile(filepath):
            try:
                with open(filepath) as f:
                    return json.load(f)
            except Exception:
                self.logger.exception('Exception on loading stored printers')
        else:
            self.logger.debug(f'No file {filepath}. That is OK')
        return []

    def save(self, printers=None, filepath=None):
        if not printers:
            printer = self.printers
        if not filepath:
            path = self.STORAGE_FILEPATH
        try:
            with open(filepath, 'w') as f:
                json.dump(f, printers)
        except Exception:
            self.logger.exception('Exception on saving stored printers')


if __name__ == "__main__":
    print(StaticDetector(None).get_printers_list())
